import json
import pathlib
import random
import string
import timeit
from functools import lru_cache, partial
from typing import Dict, List, Tuple

VOWELS = "aeiou"
CONSONANTS = [letter for letter in string.ascii_lowercase if letter not in VOWELS]


def is_letter_selection_good(required: str, optional: List[str]):
    scorebook = get_words_with_letters(
        required=required, optional="".join(optional), min_size=4
    )
    n_words = len(list(scorebook.keys()))
    n_pangrams = len([0 for v in scorebook.values() if v[0] == "Pangram!"])
    print(f"Checking set with {n_words=} and {n_pangrams=}")
    if n_words > 10 and n_pangrams >= 1 and n_words < 100:
        return True

    return False


def randomize_letters() -> Tuple[str, List[str]]:
    for _ in range(1000):
        n_vowels = random.choice([2, 3])
        out = [
            *random.sample(VOWELS, n_vowels),
            *random.sample(CONSONANTS, 7 - n_vowels),
        ]
        random.shuffle(out)
        center = out[0]
        outer = out[1:]
        if is_letter_selection_good(center, outer):
            return center, outer

    return "?", ["?", "?", "?", "?", "?", "?"]


@lru_cache()
def get_popular_words() -> List[str]:
    # Borrowed from
    # https://raw.githubusercontent.com/lzha97/spelling_bee/master/words.json
    with open(pathlib.Path(__file__).parent / "word_list.json", "r") as jsonfile:
        return json.load(jsonfile)


def pangram(s):
    return len(set(s.lower())) >= 7


def get_status_from_point_percent(percent: int) -> Tuple[str, int]:
    if percent < 2:
        return "Beginner", 0
    if percent < 5:
        return "Good Start", 1
    if percent < 8:
        return "Moving Up", 2
    if percent < 15:
        return "Good", 3
    if percent < 25:
        return "Solid", 4
    if percent < 40:
        return "Nice", 5
    if percent < 50:
        return "Great", 6
    if percent < 70:
        return "Amazing", 7
    if percent < 100:
        return "Genius", 8
    return "Queen Bee", 8


def get_word_result(word: str) -> Tuple[str, int]:
    if len(word) < 4:
        return "", 0
    if len(word) == 4:
        return "Good!", 1
    if len(word) < 7:
        return "Nice!", len(word)
    if len(set(word)) < 7:
        return "Awesome!", len(word)
    return "Pangram!", len(word) + 7


@lru_cache(maxsize=1024)
def get_words_with_letters(
    required: str, optional: str, min_size: int
) -> Dict[str, Tuple[str, int]]:
    required = required.strip().lower()
    optional = optional.strip().lower()
    all_possible_letters = required + optional
    out = {}
    for word in get_popular_words():
        if len(word) < min_size:
            continue
        if any((letter not in word for letter in required)):
            continue
        if any((letter not in all_possible_letters for letter in word)):
            continue
        out[word] = get_word_result(word)
    return out


if __name__ == "__main__":
    from rich import print

    print(
        timeit.timeit(
            partial(get_words_with_letters, required="B", optional="AILNPT", min_size=4)
        )
    )
    print(
        timeit.timeit(
            partial(get_words_with_letters, required="B", optional="AILNPT", min_size=4)
        )
    )
    print(
        timeit.timeit(
            partial(get_words_with_letters, required="B", optional="AILNPT", min_size=4)
        )
    )
    print(
        timeit.timeit(
            partial(get_words_with_letters, required="B", optional="AILNPT", min_size=4)
        )
    )
    print("labia" in get_popular_words())
    print(get_words_with_letters(required="B", optional="AILNPT", min_size=4))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['textual_bee']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'textual>=0.9.1,<0.10.0']

entry_points = \
{'console_scripts': ['textual-bee = textual_bee:run_app']}

setup_kwargs = {
    'name': 'textual-bee',
    'version': '1.1.0',
    'description': 'A Spelling Bee game for your terminal!',
    'long_description': "# textual-bee\n\n![preview](https://raw.githubusercontent.com/torshepherd/textual-bee/main/preview.png)\n\nGet it on PyPI:\n\n```sh\npip install textual-bee\ntextual-bee\n```\n\n## How to play\n\nSubmit words that include the center letter and are at least 4 letters long.\n\n- Press the shuffle button (spacebar) to shuffle the outer letters.\n- Press CTRL-C to quit.\n- Press CTRL-R to reset and choose a new set of letters.\n- Press (tab) to view your already-found words.\n\n## Scoring\n\nYour grade is based on how many of the possible words you find:\n\n| Words found | Grade      |\n| ----------- | ---------- |\n| 0-2%        | Beginner   |\n| 2-5%        | Good Start |\n| 5-8%        | Moving Up  |\n| 8-15%       | Good       |\n| 15-25%      | Solid      |\n| 25-40%      | Nice       |\n| 40-50%      | Great      |\n| 50-70%      | Amazing    |\n| 70-100%     | Genius     |\n| 100%        | Queen Bee  |\n\n### Notes:\n\nThe word list doesn't correspond exactly to Sam's, but it was the closest I could find (margin of error seemed to be about 10%). In general, this game allows more proper nouns and fewer esoteric words, but it varies depending on the setup.\n\n_Inspired by https://github.com/ajeetdsouza/clidle_\n",
    'author': 'torshepherd',
    'author_email': 'tor.aksel.shepherd@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
